package img;

/* This class demostrates how to save into .bmp file any  256 colors picture
 * represented  by a 2D integer array and a 256 color palette.
 *
 * You can use/edit either the built in palette or load any other palette
 * from the disk. Palette disk format is rgb components of particular color
 * specified on separate lines, rgb components are separated by spaces.
 *
 * See the documentation of java.awt.BufferedImage and javax.imageio.ImageIO
 * for further details on manipulating pictures.
*/

import java.util.StringTokenizer;
import java.io.*;
import java.awt.image.*;
import javax.imageio.ImageIO;

public class Save {

// ************************************************************************
//            Demonstration of simple image processing
// ************************************************************************

public static void main(String[] args) {

  // create an image
  int imgWidth = 240;
  int imgHeight = 180;
  int [][] myImage = new int [imgHeight][imgWidth];

  // fill image with some stuff
  for(int y  = 0; y < imgHeight; y++)
    for(int x = 0; x < imgWidth; x++) {
      myImage[y][x] = ((x*x+y*y)/128) % 256; // calculate pixel color somehow
  }

  // save to .bmp
  saveImage(myImage, myPalette, "myPicture.bmp");

  // or alternatively:
  int [][] myOtherPalette = loadPalette("grey.pal");
  saveImage(myImage, myOtherPalette, "myOtherPicture.bmp");
}

// ************************************************************************
//            Methods and example palette
// ************************************************************************

static int [][] loadPalette(String paletteFileName) {
  int [][] palette = new int [256][3];
  try {
  BufferedReader br = new BufferedReader(new FileReader(paletteFileName));
  StringTokenizer st;
  for (int i = 0; i < palette.length; i++) {
     st = new StringTokenizer(br.readLine());
     palette[i][0] = Integer.valueOf(st.nextToken());
     palette[i][1] = Integer.valueOf(st.nextToken());
     palette[i][2] = Integer.valueOf(st.nextToken());
     }
  br.close();
  }
  catch (IOException ee) { palette = null; }
  return palette;
}

static void saveImage(int [][] img, int [][] palette, String fname){
  int rgb;      // rgb components stored in a single int
  int icolor;   // 0 <= icolor <= 255
  int imgHeight = img.length;
  int imgWidth = img[0].length;

  try {
    // the image which will go to disk
    BufferedImage buffImg =  new BufferedImage(imgWidth, imgHeight, BufferedImage.TYPE_INT_RGB);

    // transfer color data
    for(int i = 0; i < imgHeight; i++)
      for (int j = 0; j < imgWidth; j++) {
        icolor = img[i][j];
        rgb = (palette[icolor][0] << 16) | (palette[icolor][1] << 8) | palette[icolor][2];
        buffImg.setRGB(j, i, rgb);  // pixel with coords x=j, y=i gets rgb color
      }
    // and store
    File outputfile = new File(fname);
    ImageIO.write(buffImg, "bmp", outputfile);  // .bmp format specified here
   }
  catch (IOException e) { /* your particular reaction */ }
  }

static int [][] myPalette = {
{0, 0, 0 },
{191, 0, 0},
{0, 191, 0},
{191, 191, 0},
{0, 0, 191},
{191, 0, 191},
{0, 191, 191},
{192, 192, 192},
{192, 220, 192},
{166, 202, 240},
{0, 0, 51},
{0, 0, 102},
{0, 0, 153},
{0, 0, 204},
{0, 0, 255},
{0, 51, 0},
{0, 51, 51},
{0, 51, 102},
{0, 51, 153},
{0, 51, 204},
{0, 51, 255},
{0, 102, 0},
{0, 102, 51},
{0, 102, 102},
{0, 102, 153},
{0, 102, 204},
{0, 102, 255},
{0, 153, 0},
{0, 153, 51},
{0, 153, 102},
{0, 153, 153},
{0, 153, 204},
{0, 153, 255},
{0, 204, 0},
{0, 204, 51},
{0, 204, 102},
{0, 204, 153},
{0, 204, 204},
{0, 204, 255},
{0, 255, 0},
{0, 255, 51},
{0, 255, 102},
{0, 255, 153},
{0, 255, 204},
{0, 255, 255},
{51, 0, 0},
{51, 0, 51},
{51, 0, 102},
{51, 0, 253},
{51, 0, 204},
{51, 0, 255},
{51, 51, 0},
{51, 51, 51},
{51, 51, 102},
{51, 51, 153},
{51, 51, 204},
{51, 51, 255},
{51, 102, 0},
{51, 102, 51},
{51, 102, 102},
{51, 102, 153},
{51, 102, 204},
{51, 102, 255},
{51, 153, 0},
{51, 153, 51},
{51, 153, 102},
{51, 153, 153},
{51, 153, 204},
{51, 153, 255},
{51, 204, 0},
{51, 204, 51},
{51, 204, 102},
{51, 204, 153},
{51, 204, 204},
{51, 204, 255},
{51, 255, 0},
{51, 255, 51},
{51, 255, 102},
{51, 255, 153},
{51, 255, 204},
{51, 255, 255},
{102, 0, 0},
{102, 0, 51},
{102, 0, 102},
{102, 0, 153},
{102, 0, 204},
{102, 0, 255},
{102, 51, 51},
{102, 51, 51},
{102, 51, 76},
{102, 51, 102},
{102, 51, 153},
{102, 51, 204},
{102, 51, 255},
{102, 102, 0},
{102, 102, 51},
{102, 102, 51},
{102, 102, 153},
{102, 102, 204},
{102, 102, 255},
{102, 153, 0},
{102, 153, 51},
{102, 153, 102},
{102, 153, 153},
{102, 153, 204},
{102, 153, 255},
{102, 204, 0},
{102, 204, 51},
{102, 204, 102},
{102, 204, 153},
{102, 204, 204},
{102, 204, 255},
{102, 255, 0},
{102, 255, 51},
{102, 255, 102},
{102, 255, 153},
{102, 255, 204},
{102, 255, 255},
{153, 0, 0},
{153, 0, 51},
{153, 0, 102},
{153, 0, 128},
{153, 0, 153},
{153, 0, 204},
{153, 0, 255},
{153, 51, 0},
{153, 51, 51},
{153, 51, 102},
{153, 51, 128},
{153, 51, 153},
{153, 51, 204},
{153, 51, 255},
{153, 102, 0},
{153, 102, 51},
{153, 102, 102},
{153, 102, 153},
{153, 102, 204},
{153, 102, 255},
{153, 153, 0},
{153, 153, 51},
{153, 153, 102},
{153, 153, 128},
{153, 153, 153},
{153, 153, 204},
{153, 153, 255},
{153, 204, 0},
{153, 204, 51},
{153, 204, 102},
{153, 204, 128},
{153, 204, 153},
{153, 204, 204},
{153, 204, 255},
{153, 255, 0},
{153, 255, 51},
{153, 255, 102},
{153, 255, 153},
{153, 255, 204},
{153, 255, 255},
{204, 0, 0},
{204, 0, 51},
{204, 0, 102},
{204, 0, 153},
{204, 0, 204},
{204, 0, 255},
{204, 51, 0},
{204, 51, 51},
{204, 51, 102},
{204, 51, 153},
{204, 51, 204},
{204, 51, 255},
{204, 102, 0},
{204, 102, 51},
{204, 102, 102},
{204, 102, 153},
{204, 102, 204},
{204, 102, 255},
{204, 153, 0},
{204, 153, 34},
{204, 153, 51},
{204, 153, 102},
{204, 153, 128},
{204, 153, 153},
{204, 153, 204},
{204, 153, 255},
{204, 204, 0},
{204, 204, 51},
{204, 204, 102},
{204, 204, 153},
{204, 204, 204},
{204, 204, 228},
{204, 204, 255},
{204, 255, 0},
{204, 255, 51},
{204, 255, 102},
{204, 255, 153},
{204, 255, 204},
{204, 255, 255},
{255, 0, 51},
{255, 0, 102},
{255, 0, 153},
{255, 0, 204},
{255, 0, 255},
{255, 51, 0},
{255, 51, 51},
{255, 51, 102},
{255, 51, 128},
{255, 51, 153},
{255, 51, 204},
{255, 51, 255},
{255, 102, 0},
{255, 102, 51},
{255, 102, 102},
{255, 102, 153},
{255, 102, 204},
{255, 102, 255},
{255, 153, 0},
{255, 153, 51},
{255, 153, 102},
{255, 153, 128},
{255, 153, 153},
{255, 153, 204},
{255, 153, 255},
{255, 204, 0},
{255, 204, 51},
{255, 204, 102},
{255, 204, 128},
{255, 204, 153},
{255, 204, 178},
{255, 204, 204},
{255, 204, 255},
{238, 238, 238},
{221, 221, 221},
{187, 187, 187},
{170, 170, 170},
{136, 136, 136},
{119, 119, 119},
{85, 85, 85},
{68, 68, 68},
{51, 51, 51},
{38, 38, 38},
{17, 17, 17},
{255, 255, 51},
{255, 255, 102},
{255, 255, 128},
{255, 255, 153},
{255, 255, 204},
{255, 251, 240},
{160, 160, 164},
{128, 128, 128},
{255, 0, 0},
{0, 255, 0},
{255, 255, 0},
{0, 0, 255},
{255, 0, 255},
{0, 255, 255},
{255, 255, 255}
};



}